library(statnet)
library(xergm)
library(dplyr)
library(texreg)
library(sna)

# This script creates and runs the Exponential Random Graph Models to test the hypotheses

# read in data
net <- readRDS(file = "BTnet.rds")

# set default values
burnin <- 10000      # MCMC burnin
sampsize <- 3000    # MCMC sample size
maxit <- 50         # number of MCMC MLE iterations
nsim <- 1000         # number of simulated networks for the GOF assessment
# cores <- 3           # number of computing cores for parallel processing
seed <- 98765        # random seed for exact reproducibility
set.seed(seed)

summary(net)


# Model without cluster effects (Model 1)

m1 <- ergm(net ~ 
           + edges 
           + b1nodematch("partyposition", diff = T, levels = 2) 
           + b1nodematch("gov", diff = T, levels = 2) 
           + b1nodematch("MdB_duration", diff = T, levels = 1) 
           + b1nodematch("obmann", diff = T, levels = 2) 
           + gwb2degree(2) 
           + b1degrange(4),
           control = control.ergm(MCMC.burnin = burnin, MCMC.samplesize = sampsize, seed = seed, MCMLE.maxit = maxit, MCMLE.steplength.margin=0)
)

summary(m1)

# Model 2: testing for similarities within clusters

m2 <- ergm(net ~ 
                    + edges
            + nodemix('b2cluster_b1party', b1levels=2)
            + nodemix('b2cluster_b1gov', b1levels=2)
            + nodemix('b2cluster_b1obmann', b1levels=2)
            + nodemix('b2cluster_b1MdB_duration', b1levels=1)
            + gwb2degree(2)
            + b1degrange(4),
            control = control.ergm(MCMC.burnin = burnin, MCMC.samplesize = sampsize, seed = seed, MCMLE.maxit = maxit, MCMLE.steplength.margin=0)
)

summary(net ~ 
                + edges
        + nodemix('b2cluster_b1party', b1levels=2)
        + nodemix('b2cluster_b1gov', b1levels=2)
        + nodemix('b2cluster_b1obmann', b1levels=2)
        + nodemix('b2cluster_b1MdB_duration', b1levels=1)
        + gwb2degree(2)
        + b1degrange(4))

mcmc.diagnostics(m2)
summary(m2)


# Model 3: testing for similarities within clusters with alternative operationalization of 
# institutional position (chair vs. obmann)

m3 <- ergm(net ~ 
                    + edges
            + nodemix('b2cluster_b1party', b1levels=2)
            + nodemix('b2cluster_b1gov', b1levels=2)
            + nodemix('b2cluster_b1chair', b1levels=2)
            + nodemix('b2cluster_b1MdB_duration', b1levels=1)
            + gwb2degree(2)
            + b1degrange(4),
            control = control.ergm(MCMC.burnin = burnin, MCMC.samplesize = sampsize, seed = seed, MCMLE.maxit = maxit, MCMLE.steplength.margin=0)
)

mcmc.diagnostics(m3)
summary(m3)


# Create nice looking output (Table 1)
htmlreg(list(m1, m2, m3), file = "BtNets.models_nodemix.html", inline.css = TRUE, doctype = TRUE, html.tag = TRUE, head.tag = TRUE, body.tag = TRUE,
        custom.coef.names=c("0_edges", "0_Party position (right)", "0_Opposition", "0_MdB membership (long)", "0_Speaker", "9_gwb2deg2", "9_b1deg5+", 
                            "1_Party position (right)", "2_Party position (right)", "3_Party position (right)", "4_Party position (right)", "5_Party position (right)", 
                            "1_Opposition", "2_Opposition", "3_Opposition", "4_Opposition", "5_Opposition", 
                            "1_Obmann", "2_Obmann", "3_Obmann", "4_Obmann", "5_Obmann",
                            "1_MdB membership (long)", "2_MdB membership (long)", "3_MdB membership (long)", "4_MdB membership (long)", "5_MdB membership (long)",
                            "1_Chair", "2_Chair", "3_Chair", "4_Chair", "5_Chair"),
        digits=3)


##############################

# Robustness tests
# Test for influrence of objective Europeanization
m4a <- ergm(net ~ 
                    + edges
            + nodemix('b2cluster_b1party', b1levels=2)
            + nodemix('b2cluster_b1gov', b1levels=2)
            + nodemix('b2cluster_b1chair', b1levels=2)
            + nodemix('b2cluster_b1MdB_duration', b1levels=1)
            + nodemix('b2cluster_b1europeanization', b1levels=1)
            + gwb2degree(2)
            + b1degrange(4),
            control = control.ergm(MCMC.burnin = burnin, MCMC.samplesize = sampsize, seed = seed, MCMLE.maxit = maxit, MCMLE.steplength.margin=0)
)

mcmc.diagnostics(m4a)
summary(m4a)

# Test for influrence of subjective Europeanization
m4b <- ergm(net ~ 
                    + edges
            + nodemix('b2cluster_b1party', b1levels=2)
            + nodemix('b2cluster_b1gov', b1levels=2)
            + nodemix('b2cluster_b1chair', b1levels=2)
            + nodemix('b2cluster_b1MdB_duration', b1levels=1)
            + nodemix('b2cluster_b1salience', b1levels=1)
            + gwb2degree(2)
            + b1degrange(4),
            control = control.ergm(MCMC.burnin = burnin, MCMC.samplesize = sampsize, seed = seed, MCMLE.maxit = maxit, MCMLE.steplength.margin=0)
)

mcmc.diagnostics(m4b)
summary(m4b)


# Test for seniority (long = 3+ WP)
m3b <- ergm(net ~ 
                    + edges
            + nodemix('b2cluster_b1party', b1levels=2)
            + nodemix('b2cluster_b1gov', b1levels=2)
            + nodemix('b2cluster_b1chair', b1levels=2)
            + nodemix('b2cluster_b1MdB_duration3', b1levels=1)
            + gwb2degree(2)
            + b1degrange(4),
            control = control.ergm(MCMC.burnin = burnin, MCMC.samplesize = sampsize, seed = seed, MCMLE.maxit = maxit, MCMLE.steplength.margin=0)
)

mcmc.diagnostics(m3b)
summary(m3b)


# Create nice looking output
htmlreg(list(m3, m3b, m4a, m4b), file = "BtNets.models_nodemix_robustnesstest.html", inline.css = TRUE, doctype = TRUE, html.tag = TRUE, head.tag = TRUE, body.tag = TRUE,
        digits=3)
