library(bipartite)
library(igraph)
library(dplyr)
library(ggplot2)


# This script prepares and runs the clustering in order to detect different information-seeking patterns
# It also outputs the data for the two graphs (Figure 1 & Figure 2)

# Read original network data
MdB_Info_Net_f4 <- readRDS(file = "MdB_Info_Net_f4.rds")

# Re-create party names from row names
f4_attributes <- list()
f4_attributes$party <- gsub("([[:digit:]]+)_(.+)", "\\2", row.names(MdB_Info_Net_f4))


# We use the LPA_wb_plus scripts developed by Stephen Beckett which 
# is availabe at https://github.com/sjbeckett/weighted-modularity-LPAwbPLUS
source("weighted-modularity-LPAwbPLUS/code/R/LPA_wb_plus.R")  #read in functions
source("weighted-modularity-LPAwbPLUS/code/R/GetModularInformation.R") #read in function for finding additional information



# Create empty list objects
mod4 <- list()
modList <- list()

# Run community detection 100 times
# Since the clustering process is not deterministic, this will ervery time slightly differing results.
# Our "best" solution has a modularity value of 0.3157482 (higher values are better)

for (i in 1:100) {
 modList$mod4[[i]] <- DIRT_LPA_wb_plus(MdB_Info_Net_f4)
 modList$modinfo4[[i]] <- GetModularInformation(MdB_Info_Net_f4,modList$mod4[[i]])
 }

## You may also or skip this lengthy process and load the pre-computed data:
compModules <- readRDS(file = "computed modules.rds")
modList$mod4 <- compModules$mod4
modList$modinfo4 <- compModules$modinfo4



# store relevant information in data frame
modinfo <- data.frame(matrix(ncol = 4, nrow = 100))
colnames(modinfo) <- c("id", 
                       "modularity_f4", 
                       "normalized_modularity_f4", 
                       "realized_modularity_f4")
for (i in 1:100) {
  modinfo$id[i] <- i
  modinfo$modularity_f4[i] <- modList$mod4[[i]]$modularity
  modinfo$normalized_modularity_f4[i] <- modList$modinfo4[[i]]$normalised_modularity
  modinfo$realized_modularity_f4[i] <- modList$modinfo4[[i]]$realized_modularity
}

# Find solutio with maximum modularity
maxID_4 <- modinfo$id[modinfo$normalized_modularity_f4 ==max(modinfo$normalized_modularity_f4)]
clust4 <- modList$mod4[[maxID_4]]


# Function to prepare cluster plot
preparePlotNet2 <- function(netm, mod, attrib) {
  
  net <- graph_from_incidence_matrix(netm, weighted = T, add.names = NULL, mode = "out", directed = T);
  V(net)$type <- c(rep(FALSE, dim(netm)[1]), rep(TRUE, dim(netm)[2]));          # Create bipartite mapping
  
  V(net)$comm <- c(mod$Row_labels, mod$Col_labels);
  V(net)$party[!V(net)$type] <- attrib$party;
  V(net)$color <- recode(V(net)$party, `CDU` = 'black', `CSU` = 'black', `SPD` = 'red', `Die Grünen` = 'green', `Die Linke`='violet', `FDP`='yellow');
  V(net)$name <- recode(V(net)$name, `MdB eP` = 'MdB own party', `MdB aP` = 'MdB other party', `MP eP/aMS` = 'MP own party/other MS', `MdEP eP` = 'MEP own party', `MdEP aP` = 'MEP other party', `Bundesminister eP` = 'Federal minister own party', `Bundesminister aP` = 'Federal minister other party', `Gewerkschaften D` = 'Trade unions D', `Gewerkschaften aMS` = 'Trade unions oter MS', `Gewerkschaften EU` = 'Trade unions EU', `NGO D` = 'NGO D', `NGO aMS` = 'NGO other MS', `NGO EU` = 'NGO EU', `Parteiführung eP` = 'Party leadership own party', `Fraktionsführung eP` = 'Chief whip own party', `Arbeitgeber D` = 'Employers D', `Arbeitgeber EU` = 'Employers EU', `Unternehmensverbände D` = 'Business associations D' , `Unternehmensverbände EU` = 'Business associations EU');
  V(net)$closeness <- centralization.closeness(net)$res;
  V(net)$betweenness <- centralization.betweenness(net)$res;
  V(net)$eigen <- centralization.evcent(net)$vector;
  V(net)$id <- 1:(dim(netm)[1]+dim(netm)[2])
  
  node_list <- get.data.frame(net, what = "vertices");
  node_list_from <- subset(node_list, type==FALSE);
  node_list_to <- subset(node_list, type==TRUE);
  
  edge_list <- get.data.frame(net, what = "edges") %>%
    inner_join(node_list %>% select(name, comm), by = c("from" = "name")) %>%
    inner_join(node_list %>% select(name, comm), by = c("to" = "name")) %>%
    mutate(group = ifelse(comm.x == comm.y, comm.x, NA) %>% factor());
  
  # Create a character vector containing every node name
  from_nodes <- sort(V(net)$name[!V(net)$type]);
  to_nodes <- sort(V(net)$name[V(net)$type]);
  
  return(list(net=net, node_list=node_list, node_list_from=node_list_from, node_list_to=node_list_to, edge_list=edge_list, from_nodes=from_nodes, to_nodes=to_nodes));
}


f4 <- preparePlotNet2(MdB_Info_Net_f4, clust4, f4_attributes)


# Export network in graphml format.
# This produces the network which will can then be layouted with
# visone (https://visone.net) to create Figure 2

write.graph(f4$net, file = "MdB_Info_Net_f4.graphml", format = "graphml")


# Create cluster plot (Figure 1)

from_order <- (f4$node_list_from %>% arrange(comm))$name
to_order <- (f4$node_list_to %>% arrange(comm))$name
from_colors <- (f4$node_list_from %>% arrange(comm))$color

plot_data <- f4$edge_list %>% mutate(
  to = factor(to, levels = to_order),
  from = factor(from, levels = from_order))


# Plot cluster to a pdf file

pdf("BtNnetCommunities4.pdf", height=10, width=7)

ggplot(plot_data, aes(x = to, y = from, fill = group)) +
  geom_raster() +
  theme_bw() +
  # Because we need the x and y axis to display every node,
  # not just the nodes that have connections to each other,
  # make sure that ggplot does not drop unused factor levels
  scale_x_discrete(drop = FALSE) +
  scale_y_discrete(drop = FALSE) +
  ggtitle("BtNet Communities 4") +
  ylab("MdBs") + xlab("Information Sources") +
  theme(
    # Rotate the x-axis lables so they are legible
    axis.text.x = element_text(angle = 90, hjust = 0),
    # Force the plot into a square aspect ratio
    aspect.ratio = 2,
    # Hide the legend (optional)
    legend.position = "none") + 
  theme(axis.text.y=element_text(colour = from_colors))

dev.off()

